/* sgddfilter.c: sg_dd output filter. convert sg_dd stdout and stderr output
                 to bad block list. version beta.00

   Copyright 2012 Akinori Furuta<afuruta@m7.dion.ne.jp>.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

   1. Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
   OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
   OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
   ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char *StrSkipSpace(char *p)
{if (!p)
	{return(p);
	}
 while (*p)
	{if (*p>' ')
		{break;
		}
	 p++;
	}
 return(p);
}

char *StrSkipComma(char *p)
{if (!p)
	{return(p);
	}
 p=StrSkipSpace(p);
 if (*p==',')
	{p++;
	}
 p=StrSkipSpace(p);
 return(p);
}

char single_block[]="unrecovered read error at";

#define LINE_BUFFER_SIZE	(0x10000)

/*! main, Do only filter from stdin to stdout.
    Convert following two lines into bad block list,
    one block number per one line.

    1. single bad block.
    >> unrecovered read error at blk=%lld, pdt=0, use zeros

    2. bad blocks in range.
    [0x%llx,0x%llx]
*/
int main(int argc, char **argv, char **env)
{/* stdin-stdout filter. */
 char		buf[LINE_BUFFER_SIZE];
 int		ret;
 char		*p;
 char		*p2;
 long long	blkno;
 long long	blkno2;

 ret=0;
 buf[sizeof(buf)-1]=0x00;
 while (fgets(buf,sizeof(buf)-1,stdin))
	{if ((buf[0]=='>') && (buf[1]=='>'))
		{if (strncasecmp(buf+3,single_block,sizeof(single_block)-1)==0)
			{p=strchr(buf,'=');
			 if ((p) && (*p=='='))
				{blkno=strtoll(p+1,NULL,0);
				 printf("%lld\n",blkno);
				}
			}
		}
	 if ((buf[0]=='[') && (buf[1]=='0'))
		{p=buf+1;
		 blkno=strtoll(p,&p2,0);
		 p=StrSkipComma(p2);
		 blkno2=strtoll(p,&p2,0);
		 fprintf(stderr, "Bad blocks in range: %lld, %lld\n",blkno,blkno2);
		 while (blkno<=blkno2)
			{printf("%lld\n",blkno);
			 blkno++;
			}
		}
	}
 return(ret);
}

